import org.junit.jupiter.api.Test;
import static org.junit.jupiter.api.Assertions.*;

class MagicSquareTest{

	// Normal case: 5x5 square should be generated with no issues
	@Test
	void GenerateMagicSquareNormal() {
		MagicSquare magicSquare = new MagicSquare(5);
		assertNotNull(magicSquare);
	}
	
	// Abnormal case: -5x-5 square is not possible since dimensions must be non-negative
	@Test
	void GenerateMagicSquareAbnormal() {
		assertThrows(NegativeArraySizeException.class, () -> {
			new MagicSquare(-5);
		});
	}
	
	// Boundary case: A 1x1 square should be generated with no isses
	@Test
	void GenerateMagicSquareBoundary() {
		MagicSquare magicSquare = new MagicSquare(1);
		assertNotNull(magicSquare);
	}
	
	// Normal case: Swap is valid (within bounds, but no wrap around)
	@Test
	void SwapElementNormal() {
		MagicSquare magicSquare = new MagicSquare(3);
		assertTrue(magicSquare.swapElement(1, 1, 3, false));
	}
	
	// Abnormal case: Swap is invalid because the direction is invalid
	@Test
	void SwapElementAbnormal() {
		MagicSquare magicSquare = new MagicSquare(3);
		assertFalse(magicSquare.swapElement(2, 2, 5, false));
	}
	
	// Boundary case: Swap is valid (wrap around - up)
	@Test
	void SwapElementBoundaryUp() {
		MagicSquare magicSquare = new MagicSquare(3);
		assertTrue(magicSquare.swapElement(0, 1, 0, false));
	}
	
	// Boundary case: Swap is valid (wrap around - down)
	@Test
	void SwapElementBoundaryDown() {
		MagicSquare magicSquare = new MagicSquare(3);
		assertTrue(magicSquare.swapElement(2, 2, 1, false));
	}
	
	// Boundary case: Swap is valid (wrap around - left)
	@Test
	void SwapElementBoundaryLeft() {
		MagicSquare magicSquare = new MagicSquare(3);
		assertTrue(magicSquare.swapElement(1, 0, 2, false));
	}
	
	// Boundary case: Swap is valid (wrap around - right)
	@Test
	void SwapElementBoundaryRight() {
		MagicSquare magicSquare = new MagicSquare(3);
		assertTrue(magicSquare.swapElement(1, 2, 3, false));
	}
	
	// Normal case: A freshly generated magic square should indeed be magic
	@Test
	void IsMagicSquareNormal() {
		MagicSquare magicSquare = new MagicSquare(3);
		assertTrue(magicSquare.isMagicSquare());
	}
	
	// Abnormal case: A magic square w/ one element swapped out of place should no longer be magic
	@Test
	void IsMagicSquareAbnormal() {
		MagicSquare magicSquare = new MagicSquare(3);
		magicSquare.swapElement(1, 0, 3, false);
		assertFalse(magicSquare.isMagicSquare());
	}
	
	// Boundary case: A 1x1 square is trivially magic
	@Test
	void IsMagicSquareBoundary() {
		MagicSquare magicSquare = new MagicSquare(1);
		assertTrue(magicSquare.isMagicSquare());
	}
	
	// Normal case: shuffling the magic square results in a square which is no longer magic
	@Test
	void ShuffleSquareNormal() {
		MagicSquare magicSquare = new MagicSquare(3);
		magicSquare.shuffleSquare();
		assertFalse(magicSquare.isMagicSquare());
 	}
}